/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * GIMP Color Manager Plug-In
 * Copyright (C) 2000, 2001  Karl Heinz Kremer <khk@khk.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "config.h"

#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <dirent.h>

#include <stdlib.h>		/*  For srand()  */
#include <time.h>		/*  For time()   */

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include <lcms/lcms.h>

#include "utils.h"
#include "interface.h"
#include "main.h"

#include "plugin-intl.h"

/*  Constants  */

#define INTENT_STR_LENGTH	(32)

#define PERCEPTUAL_NAME         _("Perceptual")
#define REL_COLORIMETRIC_NAME   _("Relative Colorimetric")
#define SATURATION_NAME         _("Saturation")
#define ABS_COLORIMETRIC_NAME   _("Absolute Colorimetric")


/*  Local function prototypes  */

static void profile_combo_cb (GtkWidget *, gpointer);
static void rendering_intent_combo_cb (GtkWidget *, gpointer);
static void ok_callback (GtkWidget * widget, gpointer data);

static void fselect_callback (GtkWidget * widget, gpointer data);

static void workspc_callback (GtkWidget * widget, gpointer data);
static void create_file_selection (void);
static void create_workspc_selection (void);
static void load_file_selection (void);
static void load_workspc_selection (void);
static void file_ok_sel (GtkWidget *, GtkFileSelection *);
static void file_cancel_sel (GtkWidget *, GtkFileSelection *);
static void ok_callback (GtkWidget *, gpointer);
static void refresh_callback (GtkWidget *, gpointer);
static void embed_profile_callback (GtkWidget *, gpointer);
static void file_ok_sel (GtkWidget *, GtkFileSelection *);
static void workspc_ok_sel (GtkWidget *, GtkFileSelection *);
static void file_cancel_sel (GtkWidget *, GtkFileSelection *);
static void set_profile_type (GtkWidget *, gpointer);
static void rescan_dir (guchar *);
static void cm_error (const char *, ...);
static int get_rendering_intent (void);
static gboolean check_for_gamut_tag(guchar * theProfileName);

/*  Local variables  */

static PlugInUIVals *ui_state = NULL;
static PlugInVals *state = NULL;

static GtkWidget *rendering_intent_combo = NULL;	/* rendeing intent selection */
static GtkWidget *in_profile_combo = NULL;		/* input profile selection */
static GtkWidget *out_profile_combo = NULL;		/* output profile selection */
static GtkWidget *display_profile_combo = NULL;	/* display profile selection */
static GtkWidget *file_sel = NULL;				/* directory selection */
static GtkWidget *workspc_sel = NULL;			/* color work space selection */
static GtkWidget *embedded_button = NULL;		/* use embedded profile check box */
static GtkWidget *preview_button = NULL;		/* preview button */
static GtkWidget *gamut_check_button = NULL;	/* use gamut alert color */
static GtkWidget *in_button = NULL;				/* apply input profile */
static GtkWidget *out_button = NULL;			/* apply output profile */
static GtkWidget *refresh_button = NULL;		/* refresh list of profiles */
static GtkWidget *workspc_button = NULL;		/* select work space */
static GtkWidget *dir_button = NULL;			/* select directory */
static GtkWidget *embed_profile_button = NULL;	/* embed current workspace profile */
static GtkWidget *profile_tbl = NULL;			/*  */

static gboolean run = FALSE;


/*  Public functions  */

gboolean
dialog (gint32 image_ID,
	GimpDrawable * drawable, PlugInVals * vals, PlugInUIVals * ui)
{
  GtkWidget *dlg = NULL;
  GtkWidget *main_vbox = NULL;
  GtkWidget *top_box = NULL;
  GtkWidget *button_box = NULL;
  GtkWidget *io_sel_box = NULL;
  GtkWidget *profile_box = NULL;
  GtkWidget *label = NULL;
  GSList *group = NULL;
  GList *glist = NULL;
#ifdef GIMP_HAVE_PARASITES
  GimpParasite *parasite;
#endif

  ui_state = ui;
  state = vals;


#ifdef GIMP_HAVE_PARASITES
  parasite = gimp_image_parasite_find (image_ID, "icc-profile");

  if (parasite)
    {
      ui->cm_have_embedded = TRUE;
    }
  else
    {
      ui->cm_have_embedded = FALSE;
    }
#endif

  gimp_ui_init (PLUGIN_NAME, TRUE);

  dlg = gimp_dialog_new (_("GIMP Color Manager"), PLUGIN_NAME,
			 gimp_standard_help_func, PLUGIN_NAME ".html",
			 GTK_WIN_POS_MOUSE,
			 FALSE, TRUE, FALSE,
			 _("Apply Profile"), ok_callback,
			 NULL, NULL, NULL, TRUE, FALSE,
			 _("Cancel"), gtk_widget_destroy,
			 NULL, 1, NULL, FALSE, TRUE, NULL);

  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  /*  Initialize tooltips  */
  gimp_help_init ();

  /* create top level container */

  main_vbox = gtk_vbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), main_vbox);

  /* create next box to hold buttons */

  top_box = gtk_hbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (top_box), 6);
  gtk_box_pack_start (GTK_BOX (main_vbox), top_box, FALSE, FALSE, 0);
  gtk_widget_show (top_box);

  /* create button box containing the radio buttons */

  button_box = gtk_hbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (button_box), 6);
  gtk_box_pack_start (GTK_BOX (main_vbox), button_box, FALSE, FALSE, 0);
  gtk_widget_show (button_box);

  /* add radio buttons for selection of input/output profile */

  io_sel_box = gtk_vbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (io_sel_box), 6);
  gtk_box_pack_start (GTK_BOX (top_box), io_sel_box, FALSE, FALSE, 0);

  io_sel_box = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (io_sel_box), 6);
  gtk_box_pack_start (GTK_BOX (top_box), io_sel_box, FALSE, FALSE, 0);
  gtk_widget_show (io_sel_box);

  label = gtk_label_new (_("Transformation:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_box_pack_start (GTK_BOX (io_sel_box), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  /* create button for radio button group */

  in_button = gtk_radio_button_new_with_label (NULL, _("Input"));
  gtk_box_pack_start (GTK_BOX (io_sel_box), in_button, FALSE, FALSE, 0);

  // create a new group, starting with the in_button
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (in_button));

  out_button = gtk_radio_button_new_with_label (group, _("Output"));
  gtk_box_pack_start (GTK_BOX (io_sel_box), out_button, FALSE, FALSE, 0);

  if (state->cm_direction == INPUT_PROFILE)
    {
      /* make the "Input" setting the default */
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (in_button), TRUE);
    }
  else
    {
      /* make the "Output" setting the default */
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (out_button), TRUE);
    }

  gtk_widget_show (in_button);
  gtk_widget_show (out_button);

  /* create a table for the profile selection */

  profile_tbl = gtk_table_new (2, 2, TRUE);

  /* vbox for rendering intent and profile selection */

  profile_box = gtk_vbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (profile_box), 0);
  gtk_box_pack_start (GTK_BOX (top_box), profile_box, FALSE, FALSE, 0);
  gtk_widget_show (profile_box);

  /* add the elements to profile_box */

  label = gtk_label_new (_("Rendering Intent:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), label, 0, 1, 0, 1);
  gtk_widget_show (label);

  rendering_intent_combo = gtk_combo_new ();

  glist = g_list_append (glist, PERCEPTUAL_NAME);
  glist = g_list_append (glist, REL_COLORIMETRIC_NAME);
  glist = g_list_append (glist, SATURATION_NAME);
  glist = g_list_append (glist, ABS_COLORIMETRIC_NAME);

  gtk_combo_set_popdown_strings (GTK_COMBO (rendering_intent_combo), glist);

  /* free the glist */
  g_list_free (glist);

  /* make the combo read only */
  gtk_entry_set_editable (GTK_ENTRY
			  (GTK_COMBO (rendering_intent_combo)->entry), FALSE);

  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), rendering_intent_combo,
			     1, 2, 0, 1);
  gtk_widget_show (rendering_intent_combo);


  label = gtk_label_new (_("Input Profile:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), label, 0, 1, 1, 2);
  gtk_widget_show (label);

  in_profile_combo = gtk_combo_new ();

  /* make the combo read only */
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (in_profile_combo)->entry),
			  FALSE);

  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), in_profile_combo, 1, 2,
			     1, 2);
  gtk_widget_show (in_profile_combo);

  label = gtk_label_new (_("Output Profile:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), label, 0, 1, 2, 3);
  gtk_widget_show (label);

  out_profile_combo = gtk_combo_new ();

  glist = NULL;

  /* make the combo read only */
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (out_profile_combo)->entry),
			  FALSE);

  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), out_profile_combo, 1, 2,
			     2, 3);
  gtk_widget_show (out_profile_combo);

  /* --- */

  display_profile_combo = gtk_combo_new ();

  /* make the combo read only */
  gtk_entry_set_editable (GTK_ENTRY
			  (GTK_COMBO (display_profile_combo)->entry), FALSE);

  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), display_profile_combo,
			     1, 2, 3, 4);
  gtk_widget_show (display_profile_combo);

  label = gtk_label_new (_("Display Profile:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), label, 0, 1, 3, 4);
  gtk_widget_show (label);

  /* --- */

  preview_button = gtk_check_button_new_with_label (_("Preview"));
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), preview_button, 0, 1, 4,
			     5);

  state->cm_preview = (state->cm_direction == OUTPUT_PROFILE);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preview_button),
				state->cm_preview);
  gtk_widget_show (preview_button);

  gamut_check_button = gtk_check_button_new_with_label (_("Gamut Check"));
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), gamut_check_button, 0,
			     1, 5, 6);

  state->cm_gamut_check = (state->cm_direction == OUTPUT_PROFILE);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gamut_check_button),
				state->cm_gamut_check);
  gtk_widget_show (gamut_check_button);

  embedded_button =
    gtk_check_button_new_with_label (_("Use Embedded Profile"));
  gtk_table_attach_defaults (GTK_TABLE (profile_tbl), embedded_button, 0, 1,
			     6, 7);
  gtk_widget_show (embedded_button);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (embedded_button),
				state->cm_use_embedded);
  gtk_widget_set_sensitive (embedded_button, ui_state->cm_have_embedded);

  gtk_box_pack_start (GTK_BOX (profile_box), profile_tbl, FALSE, FALSE, 0);
  gtk_widget_show (profile_tbl);

  /* create the buttons in the button box */

  dir_button = gtk_button_new_with_label (_("Set ICC Directory"));
  gtk_box_pack_start (GTK_BOX (button_box), dir_button, FALSE, FALSE, 10);
  gtk_widget_show (dir_button);

  workspc_button = gtk_button_new_with_label (_("Work Space Profile"));
  gtk_box_pack_start (GTK_BOX (button_box), workspc_button, FALSE, FALSE, 10);
  gtk_widget_show (workspc_button);

  refresh_button = gtk_button_new_with_label (_("Refresh"));
  gtk_box_pack_start (GTK_BOX (button_box), refresh_button, FALSE, FALSE, 10);
  gtk_widget_show (refresh_button);

  embed_profile_button = gtk_button_new_with_label (_("Embed Profile"));
  gtk_box_pack_start (GTK_BOX (button_box), embed_profile_button, FALSE,
		      FALSE, 10);
  gtk_widget_show (embed_profile_button);


  gtk_widget_show (GTK_DIALOG (dlg)->vbox);

  create_file_selection ();
  create_workspc_selection ();

  rescan_dir (state->cm_directory);


  /*  Show the main containers  */
  gtk_widget_show (main_vbox);
  gtk_widget_show (dlg);


  /* hide the preview checkmark if the input direction is selected */

  if (state->cm_direction == INPUT_PROFILE)
    {
      gtk_widget_set_sensitive (preview_button, FALSE);
      gtk_widget_set_sensitive (gamut_check_button, FALSE);
    }

  /* -------------------------------------------------------------------------- *
   * setup the callbacks
   * -------------------------------------------------------------------------- */

  gtk_signal_connect (GTK_OBJECT (in_button), "toggled",
		      GTK_SIGNAL_FUNC (set_profile_type),
		      (gpointer) INPUT_PROFILE);

  gtk_signal_connect (GTK_OBJECT (out_button), "toggled",
		      GTK_SIGNAL_FUNC (set_profile_type),
		      (gpointer) OUTPUT_PROFILE);

  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (rendering_intent_combo)->entry),
		      "changed", GTK_SIGNAL_FUNC (rendering_intent_combo_cb),
		      NULL);

  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (in_profile_combo)->entry),
		      "changed", GTK_SIGNAL_FUNC (profile_combo_cb),
		      state->cm_input_profile);

  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (out_profile_combo)->entry),
		      "changed", GTK_SIGNAL_FUNC (profile_combo_cb),
		      state->cm_output_profile);

  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (display_profile_combo)->entry),
		      "changed", GTK_SIGNAL_FUNC (profile_combo_cb),
		      state->cm_display_profile);

  gtk_signal_connect (GTK_OBJECT (preview_button), "toggled",
		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
		      &state->cm_preview);

  gtk_signal_connect (GTK_OBJECT (gamut_check_button), "toggled",
		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
		      &state->cm_gamut_check);

  gtk_signal_connect (GTK_OBJECT (dir_button), "clicked",
		      GTK_SIGNAL_FUNC (fselect_callback),
		      GTK_DIALOG (dlg)->vbox);

  gtk_signal_connect (GTK_OBJECT (workspc_button), "clicked",
		      GTK_SIGNAL_FUNC (workspc_callback),
		      GTK_DIALOG (dlg)->vbox);

  gtk_signal_connect (GTK_OBJECT (refresh_button), "clicked",
		      GTK_SIGNAL_FUNC (refresh_callback), NULL);

  gtk_signal_connect (GTK_OBJECT (embed_profile_button), "clicked",
		      GTK_SIGNAL_FUNC (embed_profile_callback),
		      (void *) image_ID);

  gtk_signal_connect (GTK_OBJECT (GTK_DIALOG (dlg)->vbox), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  gtk_signal_connect (GTK_OBJECT (embedded_button), "toggled",
		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
		      &state->cm_use_embedded);

  if (state->cm_direction == INPUT_PROFILE)
    {
      /* make the "Input" setting the default */
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (in_button), TRUE);
    }
  else
    {
      /* make the "Output" setting the default */
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (out_button), TRUE);
    }

  gtk_main ();

  /*  Free tooltips  */
  gimp_help_free ();

  return run;
}


/*  Private functions  */

static void
set_profile_type (GtkWidget * widget, gpointer data)
{
  state->cm_direction = (int) data;

  if (state->cm_direction == INPUT_PROFILE)
    {
      /* disable the preview checkbox */
      gtk_widget_set_sensitive (preview_button, FALSE);
      gtk_widget_set_sensitive (gamut_check_button, FALSE);
    }
  else
    {
      gtk_widget_set_sensitive (preview_button, TRUE);
      gtk_widget_set_sensitive (gamut_check_button, TRUE);

    }

  rescan_dir (state->cm_directory);
}


static void
ok_callback (GtkWidget * widget, gpointer data)
{
  run = TRUE;
  gtk_widget_destroy (GTK_WIDGET (data));
}


static void
refresh_callback (GtkWidget * widget, gpointer data)
{

  /*
   * get the directory from the file selection widget and rescan the directory
   */

  rescan_dir (state->cm_directory);
}


static void
embed_profile_callback (GtkWidget * widget, gpointer data)
{
  embed_profile ((gint32) data, state->cm_work_space);
  // activate the "embed profile" check box
  gtk_widget_set_sensitive (embedded_button, TRUE);
}



static void
create_workspc_selection (void)
{
  if (!workspc_sel)
    {
      workspc_sel = gtk_file_selection_new (_("Select Work Space Profile"));

      gtk_file_selection_set_filename (GTK_FILE_SELECTION (workspc_sel),
				       state->cm_work_space);

      gtk_signal_connect (GTK_OBJECT (workspc_sel), "destroy",
			  GTK_SIGNAL_FUNC (gtk_widget_destroyed),
			  &workspc_sel);

      gtk_signal_connect (GTK_OBJECT
			  (GTK_FILE_SELECTION (workspc_sel)->ok_button),
			  "clicked", GTK_SIGNAL_FUNC (workspc_ok_sel),
			  GTK_OBJECT (workspc_sel));

      gtk_signal_connect (GTK_OBJECT
			  (GTK_FILE_SELECTION (workspc_sel)->cancel_button),
			  "clicked", GTK_SIGNAL_FUNC (file_cancel_sel),
			  GTK_OBJECT (workspc_sel));

      gtk_widget_hide (workspc_sel);
    }
}


static void
create_file_selection (void)
{
  if (!file_sel)
    {
      file_sel = gtk_file_selection_new (_("Select ICC Profile Directory"));

      gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel),
				       state->cm_directory);

      /* hide the file selection widgets, leave just the directory stuff */

      gtk_widget_hide (GTK_FILE_SELECTION (file_sel)->fileop_del_file);
      gtk_widget_hide (GTK_FILE_SELECTION (file_sel)->fileop_ren_file);
      gtk_widget_hide (GTK_FILE_SELECTION (file_sel)->fileop_c_dir);
      gtk_widget_hide (GTK_FILE_SELECTION (file_sel)->file_list->parent);

      gtk_window_position (GTK_WINDOW (file_sel), GTK_WIN_POS_NONE);

      gtk_signal_connect (GTK_OBJECT (file_sel), "destroy",
			  GTK_SIGNAL_FUNC (gtk_widget_destroyed), &file_sel);

      gtk_signal_connect (GTK_OBJECT
			  (GTK_FILE_SELECTION (file_sel)->ok_button),
			  "clicked", GTK_SIGNAL_FUNC (file_ok_sel),
			  GTK_OBJECT (file_sel));

      gtk_signal_connect (GTK_OBJECT
			  (GTK_FILE_SELECTION (file_sel)->cancel_button),
			  "clicked", GTK_SIGNAL_FUNC (file_cancel_sel),
			  GTK_OBJECT (file_sel));

      gtk_widget_hide (file_sel);

    }
}


static void
fselect_callback (GtkWidget * widget, gpointer data)
{
  load_file_selection ();
}

static void
workspc_callback (GtkWidget * widget, gpointer data)
{
  load_workspc_selection ();
}

static void
load_workspc_selection (void)
{
  /* Load a single object */
  if (!GTK_WIDGET_VISIBLE (workspc_sel))
    gtk_widget_show (workspc_sel);
}

static void
rendering_intent_combo_cb (GtkWidget * widget, gpointer data)
{
  /*
   * get the rendering intent value from the combo box and
   * store the value to rendering_intent / the string is
   * converted in get_rendering_intent () to an int.
   */

  state->cm_rendering_intent = get_rendering_intent ();

}

static void
profile_combo_cb (GtkWidget * widget, gpointer data)
{
  strncpy ((guchar *) data,
	   gtk_entry_get_text (GTK_ENTRY (widget)), MAXPATHLEN);

  if ((data == state->cm_output_profile)
      && state->cm_direction == OUTPUT_PROFILE)
    {
      gtk_widget_set_sensitive (gamut_check_button,
				check_for_gamut_tag ((guchar *) data));
    }
}

static void
load_file_selection (void)
{
  /* Load a single object */
  if (!GTK_WIDGET_VISIBLE (file_sel))
    gtk_widget_show (file_sel);
}

static void
workspc_ok_sel (GtkWidget * w, GtkFileSelection * fs)
{
  /* get the selected file name */
  strncpy (state->cm_work_space,
	   gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)),
	   MAXPATHLEN);

  gtk_widget_hide (GTK_WIDGET (fs));
}

static void
file_ok_sel (GtkWidget * w, GtkFileSelection * fs)
{
  /* get the selected file name */
  strncpy (state->cm_directory,
	   gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)),
	   MAXPATHLEN);

  rescan_dir (state->cm_directory);

  gtk_widget_hide (GTK_WIDGET (fs));

}


static void
file_cancel_sel (GtkWidget * w, GtkFileSelection * fs)
{
  gtk_widget_hide (GTK_WIDGET (fs));
}


static void
rescan_dir (guchar * directory)
{
  struct stat statbuf;
  DIR *dir;
  struct dirent *entry;
  GList *output_list = NULL;
  GList *input_list = NULL;
  guchar *tmp_input;
  guchar *tmp_output;

  if ((lstat (directory, &statbuf)) != 0)
    {
      cm_error ("%s\n", _("Can not stat directory %s"), directory);
      strcpy (state->cm_directory, "/");
      if ((lstat (state->cm_directory, &statbuf)) != 0)
	return;
    }


  /* store the current selections in temp variables so that we 
     can restore them after updating the combo boxes */
  tmp_input = strdup (state->cm_input_profile);
  tmp_output = strdup (state->cm_output_profile);

  if (S_ISDIR (statbuf.st_mode))
    {
      dir = opendir (directory);

      if (!dir)
	{
	  cm_error ("%s\n", _("Can not open directory %s"), directory);
	  free (tmp_input);
	  free (tmp_output);
	  return;
	}

      while ((entry = readdir (dir)))
	{
	  char newfile[MAXPATHLEN];
	  char *name = entry->d_name;
	  cmsHPROFILE profile;

	  if ((strcmp (name, "..") == 0) || (strcmp (name, ".") == 0))
	    {
	      continue;
	    }

	  sprintf (newfile, "%s/%s", directory, name);

	  /*
	   * Open the file and try to read it using the lcms library
	   * - if this fails it was not a valid profile.
	   */

	  profile = cmsOpenProfileFromFile (newfile, "r");

	  if (profile != NULL)
	    {
	      char *strPtr;

	      if ((cmsGetDeviceClass (profile) == icSigOutputClass) &&
		  (cmsGetColorSpace (profile) == icSigRgbData))
		{
		  strPtr = strdup (name);
		  output_list = g_list_append (output_list, strPtr);
		}
	      else if ((cmsGetDeviceClass (profile) == icSigInputClass) &&
		       (cmsGetColorSpace (profile) == icSigRgbData))
		{
		  strPtr = strdup (name);
		  input_list = g_list_append (input_list, strPtr);
		}
	    }
	  else
	    {
	      /* not able to open profile - just ignore it for now */
	    }

	  cmsCloseProfile (profile);

	}

      // make sure that we have at least one element in the list(s)
      if (input_list == NULL)
	input_list = g_list_append (input_list, strdup (""));
      if (output_list == NULL)
	output_list = g_list_append (output_list, strdup (""));

      gtk_combo_set_popdown_strings (GTK_COMBO (in_profile_combo),
				     input_list);
      gtk_combo_set_popdown_strings (GTK_COMBO (out_profile_combo),
				     output_list);

      /* set the current selection in the lists */

      if (g_list_find_custom (input_list, tmp_input, (GCompareFunc) strcmp) !=
	  NULL)
	{
	  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (in_profile_combo)->entry),
			      tmp_input);
	}
      else
	{
	  // copy the current selection to state->cm_input_profile
	  strncpy (state->cm_input_profile,
		   gtk_entry_get_text (GTK_ENTRY
				       (GTK_COMBO (in_profile_combo)->entry)),
		   MAXPATHLEN);
	}
      if (g_list_find_custom (output_list, tmp_output, (GCompareFunc) strcmp)
	  != NULL)
	{
	  gtk_entry_set_text (GTK_ENTRY
			      (GTK_COMBO (out_profile_combo)->entry),
			      tmp_output);
	}
      else
	{
	  // copy the current selection to state->cm_output_profile
	  strncpy (state->cm_output_profile,
		   gtk_entry_get_text (GTK_ENTRY
				       (GTK_COMBO (out_profile_combo)->
					entry)), MAXPATHLEN);
	}

      gtk_widget_show (in_profile_combo);
      gtk_widget_show (out_profile_combo);
      closedir (dir);

      /* free the input_list elements and the list */
      g_list_foreach (input_list, (GFunc) g_free, NULL);
      g_list_free (input_list);

      /* free the output_list elements and the list */
      g_list_foreach (output_list, (GFunc) g_free, NULL);
      g_list_free (output_list);
    }
  else
    {
      cm_error ("%s\n", _("Not a directory: %s"), directory);
    }
  free (tmp_input);
  free (tmp_output);
}


static void
cm_error (const char *fmt, ...)
{
  va_list ap;
  guchar str[MAXPATHLEN];

  va_start (ap, fmt);
  vsnprintf (str, MAXPATHLEN, fmt, ap);
  va_end (ap);

  gimp_message (str);
}

static int
get_rendering_intent (void)
{
  char intent[INTENT_STR_LENGTH];
  int result;

  strncpy (intent,
	   gtk_entry_get_text (GTK_ENTRY
			       (GTK_COMBO (rendering_intent_combo)->entry)),
	   INTENT_STR_LENGTH);

  if (strncmp (intent, PERCEPTUAL_NAME, INTENT_STR_LENGTH) == 0)
    result = INTENT_PERCEPTUAL;
  else if (strncmp (intent, REL_COLORIMETRIC_NAME, INTENT_STR_LENGTH) == 0)
    result = INTENT_RELATIVE_COLORIMETRIC;
  else if (strncmp (intent, SATURATION_NAME, INTENT_STR_LENGTH) == 0)
    result = INTENT_SATURATION;
  else if (strncmp (intent, ABS_COLORIMETRIC_NAME, INTENT_STR_LENGTH) == 0)
    result = INTENT_ABSOLUTE_COLORIMETRIC;
  else
    result = INTENT_PERCEPTUAL;

  return result;
}

static gboolean
check_for_gamut_tag (guchar * theProfileName)
{
  cmsHPROFILE profile;
  int retVal = FALSE;
  guchar fileName[MAXPATHLEN];

  sprintf (fileName, "%s/%s", state->cm_directory, theProfileName);

  profile = cmsOpenProfileFromFile (fileName, "r");
  if (profile != NULL)
    {
      guchar *strPtr;
      retVal = cmsIsTag (profile, icSigGamutTag);
    }
  cmsCloseProfile (profile);
  return retVal;
}
