/* The GIMP -- an image manipulation program
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * GIMP Color Manager Plug-In
 * Copyright (C) 2000, 2001  Karl Heinz Kreme <khk@khk.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "config.h"

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include <lcms/lcms.h>

#include "interface.h"
#include "main.h"
#include "render.h"

#include "plugin-intl.h"


/*  Constants  */

#define PROCEDURE_NAME "gimp_color_manager"

#define DATA_KEY_VALS  "plug_in_color_manager"
#define DATA_KEY_UI    "plug_in_color_manager_ui"


/*  Local function prototypes  */

static void query (void);
static void run (gchar * name,
		 gint nparams,
		 GimpParam * param,
		 gint * nreturn_vals, GimpParam ** return_vals);
static void read_gimprc (void);
static void store_gimprc (void);


/*  Local variables  */

const PlugInVals default_vals = {
  INPUT_PROFILE,
  "/usr/lib/color/",
  "scanner.icm",
  "printer.icm",
  "display.icm",
  "/usr/lib/color/sRGB.icm",
  INTENT_PERCEPTUAL,
  0,				/* preview */
  1,				/* use embedded */
  0,				/* show gamut */
};

const PlugInUIVals default_ui = {
  0,
};

static PlugInVals vals;
static PlugInUIVals ui;


GimpPlugInInfo PLUG_IN_INFO = {
  NULL,				/* init_proc  */
  NULL,				/* quit_proc  */
  query,			/* query_proc */
  run,				/* run_proc   */
};

MAIN ()
     static void query (void)
{
  static GimpParamDef args[] = {
    {GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive"},
    {GIMP_PDB_IMAGE, "image", "Input image"},
    {GIMP_PDB_DRAWABLE, "drawable", "Input drawable"},
    {GIMP_PDB_INT32, "cm_direction",
     "Transformation direction - Input=0, Output=1"},
    {GIMP_PDB_STRING, "cm_directory", "Directory containing profiles"},
    {GIMP_PDB_STRING, "cm_input_profile",
     "Input profile to use for transformation"},
    {GIMP_PDB_STRING, "cm_output_profile",
     "Output profile to use for transformation"},
    {GIMP_PDB_STRING, "cm_display_profile",
     "Display profile to use for transformation"},
    {GIMP_PDB_STRING, "cm_work_space", "Work space color system profile"},
    {GIMP_PDB_INT32, "cm_rendering_intent", "Rendering intent"},
    {GIMP_PDB_INT32, "cm_preview", "Preview function"},
    {GIMP_PDB_INT32, "cm_use_embedded", "Use embedded Profiles"},
    {GIMP_PDB_INT32, "cm_gamut_check", "Perform gamut check"},
  };
  static gint nargs = sizeof (args) / sizeof (args[0]);

  gimp_plugin_domain_register (PLUGIN_NAME, LOCALEDIR);
  gimp_plugin_help_register (DATADIR "/help");

  gimp_install_procedure (PROCEDURE_NAME,
			  "Apply a color transformation based on ICC profiles to an image",
			  "Help!",
			  "Karl Heinz Kremer <khk@khk.net>",
			  "Karl Heinz Kremer <khk@khk.net>",
			  "2000-05-18",
			  N_("<Image>/Filters/Color Manager ..."),
			  "RGB*, GRAY*, INDEXED*",
			  GIMP_PLUGIN, nargs, 0, args, NULL);
}

static void
run (gchar * name,
     gint n_params,
     GimpParam * param, gint * nreturn_vals, GimpParam ** return_vals)
{
  static GimpParam values[1];
  GimpDrawable *drawable;
  gint32 image_ID;
  gint32 drawable_ID;
  GimpRunModeType run_mode;
  GimpPDBStatusType status = GIMP_PDB_SUCCESS;

  *nreturn_vals = 1;
  *return_vals = values;

  run_mode = param[0].data.d_int32;
  image_ID = param[1].data.d_int32;
  drawable_ID = param[2].data.d_drawable;
  drawable = gimp_drawable_get (drawable_ID);

  /*  Initialize with default values  */
  vals = default_vals;
  ui = default_ui;

  if (strcmp (name, PROCEDURE_NAME) == 0)
    {

//      read_gimprc ();

      cmsErrorAction (LCMS_ERROR_SHOW);

      switch (run_mode)
	{
	case GIMP_RUN_NONINTERACTIVE:
	  if (n_params != 11)
	    {
	      status = GIMP_PDB_CALLING_ERROR;
	    }
	  else
	    {
	      vals.cm_direction = (gint) param[3].data.d_int32;
	      strncpy (vals.cm_directory, (guchar *) param[4].data.d_string,
		       MAXPATHLEN);
	      strncpy (vals.cm_input_profile,
		       (guchar *) param[5].data.d_string, MAXPATHLEN);
	      strncpy (vals.cm_output_profile,
		       (guchar *) param[6].data.d_string, MAXPATHLEN);
	      strncpy (vals.cm_display_profile,
		       (guchar *) param[7].data.d_string, MAXPATHLEN);
	      strncpy (vals.cm_work_space, (guchar *) param[8].data.d_string,
		       MAXPATHLEN);
	      vals.cm_rendering_intent = (gint) param[9].data.d_int32;
	      vals.cm_preview = (gint) param[10].data.d_int32;
	      vals.cm_use_embedded = (gint) param[11].data.d_int32;
	      vals.cm_gamut_check = (gint) param[12].data.d_int32;
	    }
	  break;

	case GIMP_RUN_INTERACTIVE:
	  /*  Possibly retrieve data  */
	  gimp_get_data (DATA_KEY_VALS, &vals);
	  gimp_get_data (DATA_KEY_UI, &ui);

	  INIT_I18N_UI ();
	  if (!dialog (image_ID, drawable, &vals, &ui))
	    {
	      status = GIMP_PDB_CANCEL;
	    }
	  break;

	case GIMP_RUN_WITH_LAST_VALS:
	  /*  Possibly retrieve data  */
	  gimp_get_data (DATA_KEY_VALS, &vals);
	  break;

	default:
	  break;
	}
    }
  else
    {
      status = GIMP_PDB_CALLING_ERROR;
    }

  if (status == GIMP_PDB_SUCCESS)
    {
      render (image_ID, drawable_ID, &vals, &ui);

      if (run_mode != GIMP_RUN_NONINTERACTIVE)
	gimp_displays_flush ();

      if (run_mode == GIMP_RUN_INTERACTIVE)
	{
	  gimp_set_data (DATA_KEY_VALS, &vals, sizeof (vals));
	  gimp_set_data (DATA_KEY_UI, &ui, sizeof (ui));
	}

      gimp_drawable_detach (drawable);
    }

//  store_gimprc ();

  values[0].type = GIMP_PDB_STATUS;
  values[0].data.d_status = status;
}



static void
store_gimprc (void)
{
  gint nreturn_vals;
  GimpParam *return_vals;
  guchar str[32];

  sprintf (str, "%d", vals.cm_direction);
  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_direction",
				    GIMP_PDB_STRING, str, GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_directory",
				    GIMP_PDB_STRING, vals.cm_directory,
				    GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_input_profile",
				    GIMP_PDB_STRING, vals.cm_input_profile,
				    GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_output_profile",
				    GIMP_PDB_STRING, vals.cm_output_profile,
				    GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_display_profile",
				    GIMP_PDB_STRING, vals.cm_display_profile,
				    GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_work_space",
				    GIMP_PDB_STRING, vals.cm_work_space,
				    GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);


  sprintf (str, "%d", vals.cm_rendering_intent);
  return_vals = gimp_run_procedure ("gimp_gimprc_set",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_rendering_intent",
				    GIMP_PDB_STRING, str, GIMP_PDB_END);
  gimp_destroy_params (return_vals, nreturn_vals);

}

static void
read_gimprc (void)
{
  gint nreturn_vals;
  GimpParam *return_vals;

  fprintf (stderr, "in read_gimprc() ... ");
  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_direction",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    vals.cm_direction = atoi (return_vals[1].data.d_string);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_directory",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    strncpy (vals.cm_directory, return_vals[1].data.d_string, MAXPATHLEN);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_input_profile",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    strncpy (vals.cm_input_profile, return_vals[1].data.d_string, MAXPATHLEN);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_output_profile",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    strncpy (vals.cm_output_profile, return_vals[1].data.d_string,
	     MAXPATHLEN);

  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_display_profile",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    strncpy (vals.cm_display_profile, return_vals[1].data.d_string,
	     MAXPATHLEN);

  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_work_space",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    strncpy (vals.cm_work_space, return_vals[1].data.d_string, MAXPATHLEN);
  gimp_destroy_params (return_vals, nreturn_vals);

  return_vals = gimp_run_procedure ("gimp_gimprc_query",
				    &nreturn_vals,
				    GIMP_PDB_STRING, "cm_rendering_intent",
				    GIMP_PDB_END);

  if (return_vals[0].data.d_status == GIMP_PDB_SUCCESS &&
      return_vals[1].data.d_string != NULL)
    vals.cm_rendering_intent = atoi (return_vals[1].data.d_string);
  gimp_destroy_params (return_vals, nreturn_vals);
  fprintf (stderr, "done\n");
}
